<?php

namespace Amasty\InvisibleCaptcha\Plugin;

use Amasty\InvisibleCaptcha\Helper\Data;
use Closure;
use Exception;
use Magento\Backend\Model\View\Result\RedirectFactory;
use Magento\Framework\App\FrontControllerInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\HTTP\Client\Curl;
use Magento\Framework\Message\ManagerInterface;
use Magento\Framework\ObjectManager\ObjectManager;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\UrlInterface;
use Magento\Framework\View\Result\PageFactory;

/**
 * @class Predispatch
 * @package Amasty\InvisibleCaptcha\Plugin
 */
class Predispatch
{

    /**
     * Google URl for checking captcha response
     */
    const GOOGLE_VERIFY_URL = 'https://www.google.com/recaptcha/api/siteverify';

    /**
     * @var RedirectFactory
     */
    protected $resultRedirectFactory;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var Curl
     */
    protected $curl;

    /**
     * @var UrlInterface
     */
    protected $urlBuilder;

    /**
     * @var ManagerInterface
     */
    protected $messageManager;

    /**
     * @var PageFactory
     */
    protected $resultPageFactory;

    /**
     * @var JsonFactory
     */
    private $resultJsonFactory;

    /**
     * @var ObjectManager
     */
    private $objectManager;

    /**
     * Action constructor.
     * @param RedirectFactory $resultRedirectFactory
     * @param Data $helper
     * @param Curl $curl
     * @param UrlInterface $urlBuilder
     * @param ManagerInterface $messageManager
     * @param JsonFactory $resultJsonFactory
     * @param PageFactory $resultPageFactory
     * @param ObjectManager $objectManager
     */
    public function __construct(
        RedirectFactory  $resultRedirectFactory,
        Data             $helper,
        Curl             $curl,
        UrlInterface     $urlBuilder,
        ManagerInterface $messageManager,
        JsonFactory      $resultJsonFactory,
        PageFactory      $resultPageFactory,
        ObjectManager    $objectManager
    ) {
        $this->resultRedirectFactory = $resultRedirectFactory;
        $this->helper = $helper;
        $this->curl = $curl;
        $this->urlBuilder = $urlBuilder;
        $this->messageManager = $messageManager;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->resultPageFactory = $resultPageFactory;
        $this->objectManager = $objectManager;
    }

    /**
     * @param FrontControllerInterface $subject
     * @param Closure $proceed
     * @param RequestInterface $request
     *
     * @return ResponseInterface|ResultInterface
     */
    public function aroundDispatch(
        FrontControllerInterface $subject,
        Closure                  $proceed,
        RequestInterface         $request
    ) {
        $validation = true;

        if ($this->helper->isModuleOn()) {
            foreach ($this->helper->getCaptchaUrls() as $captchaUrl) {
                if (strpos($this->urlBuilder->getCurrentUrl(), $captchaUrl) !== false) {
                    if ($request->isPost()) {

                        $token = $request->getPost('amasty_invisible_token');
                        $validation = $this->verifyCaptcha($token);

                        if (!$validation) {

                            if ($request->isXmlHttpRequest()) {
                                $resultJson = $this->resultJsonFactory->create();
                                $resultJson->setData(['success' => false, 'error' => true, 'message' => __('Incorrect Recaptcha.')]);
                                return $resultJson;
                            } else {
                                $resultPage = $this->resultPageFactory->create();
                                $this->messageManager->addErrorMessage(__('Incorrect Recaptcha.'));
                                return $resultPage;
                            }

                        }
                    }

                    break;
                }
            }
        }

        if ($validation) {
            $result = $proceed($request);
        } else {
            if ($request->isXmlHttpRequest()) {
                $resultJson = $this->resultJsonFactory->create();
                $resultJson->setData(['success' => false, 'error' => true, 'message' => __('Incorrect Recaptcha.')]);
                return $resultJson;
            } else {
                $resultPage = $this->resultPageFactory->create();
                $this->messageManager->addErrorMessage(__('Incorrect Recaptcha.'));
                return $resultPage;
            }
        }

        return $result;
    }

    /**
     * @param string $token
     * @return bool
     */
    protected function verifyCaptcha($token)
    {
        if ($token) {
            $curlParams = [
                'secret' => $this->helper->getConfigValueByPath(Data::CONFIG_PATH_GENERAL_SECRET_KEY),
                'response' => $token
            ];
            $this->curl->setOption(CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
            $this->curl->post(self::GOOGLE_VERIFY_URL, $curlParams);
            try {

                if (($this->curl->getStatus() == 200)
                    && array_key_exists('success', $answer = $this->decode($this->curl->getBody()))
                ) {

                    if ($answer['success'] == '') {
                        $answer['success'] = '0';
                    }
                    return $answer['success'];
                }
            } catch (Exception $e) {
                return false;
            }
        }

        return false;
    }

    /**
     * @param string $json
     * @return mixed|void
     */
    public function decode(string $json)
    {
        if (class_exists(Json::class)) {
            $jsonSerializer = $this->objectManager->create(Json::class);
            return $jsonSerializer->unserialize($json);
        }

        return json_decode($json, true);
    }
}
